@@ -78,7 +78,10 @@ let rec simplify min max = function
 let segments_of_partition p =
   let seg = ref [] in
   Array.iteri
-    (fun i c -> List.iter (fun (a, b) -> seg := (a, b, i) :: !seg) c)
+    (fun i c ->
+      List.iter
+        (fun (a, b) -> seg := (a, b, i) :: !seg)
+        (c : Sedlex_cset.t :> (int * int) list))
     p;
   List.sort (fun (a1, _, _) (a2, _, _) -> compare a1 a2) !seg
 
@@ -111,14 +114,14 @@ let builtin_regexps =
     ([
        ("any", Cset.any);
        ("eof", Cset.eof);
-       ("xml_letter", Cset.letter);
-       ("xml_digit", Cset.digit);
-       ("xml_extender", Cset.extender);
-       ("xml_base_char", Cset.base_char);
-       ("xml_ideographic", Cset.ideographic);
-       ("xml_combining_char", Cset.combining_char);
-       ("xml_blank", Cset.blank);
-       ("tr8876_ident_char", Cset.tr8876_ident_char);
+       ("xml_letter", Xml.letter);
+       ("xml_digit", Xml.digit);
+       ("xml_extender", Xml.extender);
+       ("xml_base_char", Xml.base_char);
+       ("xml_ideographic", Xml.ideographic);
+       ("xml_combining_char", Xml.combining_char);
+       ("xml_blank", Xml.blank);
+       ("tr8876_ident_char", Iso.tr8876_ident_char);
      ]
     @ Unicode.Categories.list @ Unicode.Properties.list)
 
@@ -177,7 +180,11 @@ let partition (name, p) =
           Char.code (String.unsafe_get [%e evar ~loc (table_name t)] [%e c]) - 1]
   in
   let body = gen_tree (simplify_decision_tree (decision_table p)) in
-  glb_value name [%expr fun c -> [%e body]]
+  glb_value name
+    [%expr
+      fun c ->
+        let open! Stdlib in
+        [%e body]]
 
 (* Code generation for the automata *)
 
@@ -196,9 +203,9 @@ let call_state lexbuf auto state =
     match best_final final with
       | Some i -> eint ~loc:default_loc i
       | None -> assert false)
-  else appfun (state_fun state) [evar ~loc:default_loc lexbuf]
+  else appfun (state_fun state) [lexbuf]
 
-let gen_state lexbuf auto i (trans, final) =
+let gen_state (lexbuf_name, lexbuf) auto i (trans, final) =
   let loc = default_loc in
   let partition = Array.map fst trans in
   let cases =
@@ -211,22 +218,21 @@ let gen_state lexbuf auto i (trans, final) =
   let body () =
     pexp_match ~loc
       (appfun (partition_name partition)
-         [[%expr Sedlexing.__private__next_int [%e evar ~loc lexbuf]]])
+         [[%expr Sedlexing.__private__next_int [%e lexbuf]]])
       (cases
       @ [
           case
             ~lhs:[%pat? _]
             ~guard:None
-            ~rhs:[%expr Sedlexing.backtrack [%e evar ~loc lexbuf]];
+            ~rhs:[%expr Sedlexing.backtrack [%e lexbuf]];
         ])
   in
   let ret body =
+    let lhs = pvar ~loc:lexbuf.pexp_loc lexbuf_name in
     [
       value_binding ~loc
         ~pat:(pvar ~loc (state_fun i))
-        ~expr:
-          (pexp_function ~loc
-             [case ~lhs:(pvar ~loc lexbuf) ~guard:None ~rhs:body]);
+        ~expr:(pexp_function ~loc [case ~lhs ~guard:None ~rhs:body]);
     ]
   in
   match best_final final with
@@ -235,7 +241,7 @@ let gen_state lexbuf auto i (trans, final) =
     | Some i ->
         ret
           [%expr
-            Sedlexing.mark [%e evar ~loc lexbuf] [%e eint ~loc i];
+            Sedlexing.mark [%e lexbuf] [%e eint ~loc i];
             [%e body ()]]
 
 let gen_recflag auto =
@@ -253,7 +259,7 @@ let gen_recflag auto =
     Nonrecursive
   with Exit -> Recursive
 
-let gen_definition lexbuf l error =
+let gen_definition ((_, lexbuf) as lexbuf_with_name) l error =
   let loc = default_loc in
   let brs = Array.of_list l in
   let auto = Sedlex.compile (Array.map fst brs) in
@@ -263,13 +269,13 @@ let gen_definition lexbuf l error =
          (fun i (_, e) -> case ~lhs:(pint ~loc i) ~guard:None ~rhs:e)
          brs)
   in
-  let states = Array.mapi (gen_state lexbuf auto) auto in
+  let states = Array.mapi (gen_state lexbuf_with_name auto) auto in
   let states = List.flatten (Array.to_list states) in
   pexp_let ~loc (gen_recflag auto) states
     (pexp_sequence ~loc
-       [%expr Sedlexing.start [%e evar ~loc lexbuf]]
+       [%expr Sedlexing.start [%e lexbuf]]
        (pexp_match ~loc
-          (appfun (state_fun 0) [evar ~loc lexbuf])
+          (appfun (state_fun 0) [lexbuf])
           (cases @ [case ~lhs:(ppat_any ~loc) ~guard:None ~rhs:error])))
 
 (* Lexer specification parser *)
@@ -427,7 +433,8 @@ let mapper =
         | [%expr [%sedlex [%e? { pexp_desc = Pexp_match (lexbuf, cases) }]]] ->
             let lexbuf =
               match lexbuf with
-                | { pexp_desc = Pexp_ident { txt = Lident lexbuf } } -> lexbuf
+                | { pexp_desc = Pexp_ident { txt = Lident txt } } ->
+                    (txt, lexbuf)
                 | _ ->
                     err lexbuf.pexp_loc
                       "the matched expression must be a single identifier"
@@ -436,7 +443,7 @@ let mapper =
             let error =
               match List.hd cases with
                 | { pc_lhs = [%pat? _]; pc_rhs = e; pc_guard = None } ->
-                    super#expression e
+                    this#expression e
                 | { pc_lhs = p } ->
                     err p.ppat_loc
                       "the last branch must be a catch-all error case"
@@ -446,7 +453,7 @@ let mapper =
               List.map
                 (function
                   | { pc_lhs = p; pc_rhs = e; pc_guard = None } ->
-                      (regexp_of_pattern env p, super#expression e)
+                      (regexp_of_pattern env p, this#expression e)
                   | { pc_guard = Some e } ->
                       err e.pexp_loc "'when' guards are not supported")
                 cases
